declare filename "snareStein.dsp"; declare name "snareStein"; // Port from SuperCollider to Faust of snare_stein in
// SynthDefPool by Dan Stowell, which in turn was based
// on a Sound-on-Sound 'synth secrets' tutorial

//https://github.com/josmithiii/faust-jos/tree/master/percussion

SR = 44100;
PI = 3.1415926535897932385;

decay(n,x) = x - (x>0.0)/n;
release(n) = + ~ decay(n);
envgate(dur,trigger) = trigger : release(int(dur*float(SR))) : >(0.0);

adsr(a,d,s,r,t) = env ~ (_,_) : (!,_) // the 2 'state' signals are fed back
with {
    env (p2,y) =
        (t>0) & (p2|(y>=1)),          // p2 = decay-sustain phase
        (y + p1*u - (p2&(y>s))*v*y - p3*w*y)	// y  = envelop signal
	*((p3==0)|(y>=eps)) // cut off tails to prevent denormals
    with {
		p1 = (p2==0) & (t>0) & (y<1);         // p1 = attack phase
		p3 = (t<=0) & (y>0);                  // p3 = release phase
		// #samples in attack, decay, release, must be >0
		na = SR*a+(a==0.0); nd = SR*d+(d==0.0); nr = SR*r+(r==0.0);
		// correct zero sustain level
		z = s+(s==0.0)*db2linear(-60);
		// attack, decay and (-60dB) release rates
		u = 1/na; v = 1-pow(z, 1/nd); w = 1-1/pow(z*db2linear(60), 1/nr);
		// values below this threshold are considered zero in the release phase
		eps = db2linear(-120);
    };
};

impulse = 1-1';
oscrs(f) = impulse : nlf2(f,1) : _,!; // sine

nlf2(f,r,x) = ((_<:_,_),(_<:_,_) : (*(s),*(c),*(c),*(0-s)) :> (*(r),+(x))) ~ cross
with {
  th = 2*PI*f/SR;
  c = cos(th);
  s = sin(th);
  cross = _,_ <: !,_,_,!;
};

perc(att,rel,trigger) = adsr(att,0.001,1.0,rel,envgate(att,trigger));
sinosc0(freq) = oscrs(freq); // SinOsc at phase fi.zero

//line(start,end,dur,trigger) = trigger : release(int(dur*float(SR))) : *(start-end)+end;
		    
lpf(freq) = lowpass(3,freq);
hpf(freq) = highpass(3,freq);

lowpass(N,fc) = lowpass0_highpass1(0,N,fc);
highpass(N,fc) = lowpass0_highpass1(1,N,fc);

lowpass0_highpass1(s,N,fc) = lphpr(s,N,N,fc)
with {
  lphpr(s,0,N,fc) = _;
  lphpr(s,1,N,fc) = tf1s(s,1-s,1,2*PI*fc);
  lphpr(s,O,N,fc) = lphpr(s,(O-2),N,fc) : tf2s(s,0,1-s,a1s,1,w1) with {
    parity = N % 2;
    S = (O-parity)/2; // current section number
    a1s = -2*cos((PI)*-1 + (1-parity)*PI/(2*N) + (S-1+parity)*PI/N);
    w1 = 2*PI*fc;
  };
};

tf1s(b1,b0,a0,w1) = tf1(b0d,b1d,a1d)
with {
  c   = 1/tan(w1*0.5/SR); // bilinear-transform scale-factor
  d   = a0 + c;
  b1d = (b0 - b1*c) / d;
  b0d = (b0 + b1*c) / d;
  a1d = (a0 - c) / d;
};

tf1(b0,b1,a1) = _ <: *(b0), (mem : *(b1)) :> + ~ *(0-a1);
tf2(b0,b1,b2,a1,a2) = iir((b0,b1,b2),(a1,a2));

iir(bv,av) = sub ~ fir(av) : fir(bv);
sub(x,y) = y-x;

fir((b0,bv)) = _ <: *(b0), R(1,bv) :> _ with {
	R(n,(bn,bv)) = (@(n):*(bn)), R(n+1,bv);
	R(n, bn)     = (@(n):*(bn));              };
fir(b0) = *(b0);

tf2s(b2,b1,b0,a1,a0,w1) = tf2(b0d,b1d,b2d,a1d,a2d)
with {
  c   = 1/tan(w1*0.5/SR); // bilinear-transform scale-factor
  csq = c*c;
  d   = a0 + a1 * c + csq;
  b0d = (b0 + b1 * c + b2 * csq)/d;
  b1d = 2 * (b0 - b2 * csq)/d;
  b2d = (b0 - b1 * c + b2 * csq)/d;
  a1d = 2 * (a0 - csq)/d;
  a2d = (a0 - a1*c + csq)/d;
};

gate = checkbox("gate");
trigger = (gate>gate');

amp = hslider("gain",-20,-60,40,0.00001) : smooth(0.999) : db2linear;

db2linear(l) = pow(10, l/20.0);

noise = random / RANDMAX
with{
	mask = 4294967295; // 2^32-1
	random = +(12345) ~ *(1103515245) & mask; // "linear congruential"
	RANDMAX	= 2147483647.0; // = 2^31-1 = MAX_SIGNED_INT in 32 bits
};

smooth(s) = *(1.0 - s) : + ~ *(s);

lpnoise = noise : lpf(7040);
hpnoise = noise : hpf(523);
att = hslider("attack",0.5, 0.,2000.,1)/1000; // attack-time in milliseconds
relfact = hslider("rel_fact",1., 0.0001,200.,0.0001); // release-time in factor

snare = (0.25 + sinosc0(330)) * perc(att,0.055*relfact,trigger)
      + (0.25 + sinosc0(185)) * perc(att,0.075*relfact,trigger)
      + 0.2 * lpnoise * perc(att,0.2*relfact,trigger)
      + 0.2 * hpnoise * perc(att,0.183*relfact,trigger);

process = snare * amp;